"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const logger_1 = require("../common/logger");
const version_1 = require("../version");
const auth_token_claims_1 = require("./auth-token-claims");
const error_code_1 = __importDefault(require("./error-code"));
const web_socket_close_reasons_1 = require("./web-socket-close-reasons");
class AuthSession {
    get namespace() {
        var _a;
        return ((_a = this.claims) === null || _a === void 0 ? void 0 : _a.auth.ans) || this.settings.namespace;
    }
    get backendOrigin() {
        var _a;
        return this.backendOriginOverride || ((_a = this.claims) === null || _a === void 0 ? void 0 : _a.auth.endp);
    }
    get authId() {
        var _a;
        return (_a = this.claims) === null || _a === void 0 ? void 0 : _a.auth.id;
    }
    get challenge() {
        var _a;
        return (_a = this.claims) === null || _a === void 0 ? void 0 : _a.auth.chlg;
    }
    get next() {
        var _a;
        return (_a = this.claims) === null || _a === void 0 ? void 0 : _a.auth.next;
    }
    constructor(settings, platform, authToken) {
        var _a, _b;
        this.channels = new Set();
        this.log = logger_1.LoggerFactory.getLogger('auth-session');
        this.platform = platform;
        this.authToken = authToken;
        this.settings = settings;
        if (authToken) {
            this.claims = this.parseJwt(authToken);
            this.settings.namespace = this.claims.auth.ans;
            this.uvLevel =
                ((_b = (_a = this.claims.auth.subs.dev) === null || _a === void 0 ? void 0 : _a.auths.pasv) === null || _b === void 0 ? void 0 : _b.uvlvl) || auth_token_claims_1.UserVerificationLevel.Discouraged;
        }
        this.requestSigner = platform.createRequestSigner(this);
    }
    fetchFromBackend(query, body) {
        const method = 'POST';
        const contentType = 'application/json';
        const bodyStr = JSON.stringify(body);
        const querySplit = query.split('?');
        const path = querySplit[0];
        const params = querySplit.length > 1 ? querySplit[1] : '';
        const headers = new Headers({
            Accept: contentType,
            'Content-Type': contentType,
            'PA-Version': version_1.USER_AGENT_VERSIONS,
        });
        if (this.authToken) {
            headers.set('Authorization', 'Bearer ' + this.authToken);
        }
        return new Promise((resolve, reject) => {
            this.requestSigner
                .sign(method, path, params, contentType, bodyStr)
                .then((signature) => {
                if (signature) {
                    headers.set('PA-Sig-Version', signature.version);
                    headers.set('PA-Challenge', signature.challenge);
                    headers.set('PA-KID', signature.keyId);
                    headers.set('PA-Signature', signature.signature);
                }
                this.platform
                    .fetch(this.backendOrigin + query, {
                    mode: 'cors',
                    method: method,
                    headers: headers,
                    body: bodyStr,
                })
                    .then((response) => response.json())
                    .then((response) => {
                    if (response.error) {
                        if (response.error.code === error_code_1.default.ERROR_NO_DEVICE_FOUND) {
                            this.settings.reset();
                            this.platform.deviceAuth.reset();
                        }
                    }
                    resolve(response);
                })
                    .catch(reject);
            })
                .catch(reject);
        });
    }
    static parseCloseEvent(event) {
        var reason = web_socket_close_reasons_1.WebSocketCloseReasons.UNKNOWN_REASON;
        switch (event.code) {
            case 1000:
                reason = web_socket_close_reasons_1.WebSocketCloseReasons.NORMAL_CLOSURE;
                break;
            case 1001:
                reason = web_socket_close_reasons_1.WebSocketCloseReasons.GOING_AWAY;
                break;
            case 1002:
                reason = web_socket_close_reasons_1.WebSocketCloseReasons.PROTOCOL_ERROR;
                break;
            case 1003:
                reason = web_socket_close_reasons_1.WebSocketCloseReasons.UNSUPPORTED_DATA;
                break;
            case 1005:
                reason = web_socket_close_reasons_1.WebSocketCloseReasons.NO_STATUS_RECEIVED;
                break;
            case 1006:
                reason = web_socket_close_reasons_1.WebSocketCloseReasons.ABNORMAL_CLOSURE;
                break;
            case 1007:
                reason = web_socket_close_reasons_1.WebSocketCloseReasons.INVALID_FRAME_PAYLOAD_DATA;
                break;
            case 1008:
                reason = web_socket_close_reasons_1.WebSocketCloseReasons.POLICY_VIOLATION;
                break;
            case 1009:
                reason = web_socket_close_reasons_1.WebSocketCloseReasons.MESSAGE_TOO_BIG;
                break;
            case 1010:
                reason = web_socket_close_reasons_1.WebSocketCloseReasons.MANDATORY_EXTENSION;
                break;
            case 1011:
                reason = web_socket_close_reasons_1.WebSocketCloseReasons.INTERNAL_ERROR;
                break;
            case 1015:
                reason = web_socket_close_reasons_1.WebSocketCloseReasons.TLS_HANDSHAKE;
                break;
        }
        return [reason, event.code];
    }
    createMessageChannel(query, onClose, onError, onMessage) {
        if (!this.authToken) {
            throw new Error('Authentication token is not initialized, cannot create MessageChannel');
        }
        const KEEP_ALIVE_INTERVAL = 30000;
        const endpoint = this.backendOrigin.replace(/^http/, 'ws');
        const channel = this.platform.createMessageChannel(endpoint + query);
        var pongReceived = true;
        const keepAlive = setInterval(() => {
            if (pongReceived) {
                this.log.trace('Sending ping message');
                pongReceived = false;
                channel.send('ping');
            }
            else {
                this.log.warn('Failed to receive ping response in time, closing the channel');
                clearInterval(keepAlive);
                channel.close();
                onError('Channel communication stalled');
                this.channels.delete(channel);
            }
        }, KEEP_ALIVE_INTERVAL);
        channel.addEventListener('close', (event) => {
            var statusCode = event.code ? `with status code: ${event.code}` : 'without status code';
            this.log.debug(`Message channel is closed ${statusCode}`);
            clearInterval(keepAlive);
            this.channels.delete(channel);
            const [reason, code] = AuthSession.parseCloseEvent(event);
            onClose(reason, code);
        });
        channel.addEventListener('error', (event) => {
            this.log.debug('Message channel encountered an error');
            this.log.debug(event);
        });
        channel.addEventListener('message', (event) => {
            this.log.trace('Message channel received a message');
            this.log.trace(event);
            if ('origin' in event && event['origin'] !== endpoint) {
                onError('Unexpected origin');
            }
            else {
                const data = event.data;
                if (data && typeof data === 'string') {
                    if (data === 'pong') {
                        pongReceived = true;
                    }
                    else {
                        onMessage(data);
                    }
                }
                else {
                    onMessage(event.toString());
                }
            }
        });
        this.channels.add(channel);
        return channel;
    }
    closeAllMessageChannels() {
        this.log.trace('Closing all message channels, total ' + this.channels.size + ' registered channels');
        const channelsCopy = new Set(this.channels);
        channelsCopy.forEach((channel) => channel.close());
        channelsCopy.clear();
        this.channels.clear();
    }
    getDeviceRegistration() {
        return new Promise((resolve, reject) => {
            this.platform.deviceAuth
                .getRegistration(this.namespace)
                .then((registration) => {
                if (registration) {
                    if (!this.backendOriginOverride) {
                        this.backendOriginOverride = registration.endpoint;
                        this.log.debug('backend URL overridden with ' + this.backendOriginOverride);
                    }
                    else {
                        this.log.debug('Not overriding backend URL since it has been already set');
                    }
                }
                resolve(registration);
            })
                .catch(reject);
        });
    }
    embedFpResultToDeviceRegistration(registration) {
        return new Promise((resolve) => {
            this.getFingerprintData()
                .then((result) => {
                if (result) {
                    registration.setFpSignal(result);
                }
                resolve(registration);
            })
                .catch((error) => {
                const errorMsg = `Unexpected error happened during Fingerprint data collection: ${error.toString}`;
                this.log.warn(errorMsg);
                registration.setFpSignal({ error: errorMsg });
            });
        });
    }
    getFingerprintData() {
        return new Promise((resolve) => {
            var fpPromise = this.platform.getFpPromise();
            if (!this.shouldCollectFP()) {
                this.log.debug('Fingerprint is not enabled from AuthToken');
                resolve(undefined);
            }
            else if (!fpPromise) {
                const msg = 'Found null instance of Fingerprint, check if your input API key is valid';
                this.log.warn(msg);
                resolve({ error: msg });
            }
            else {
                fpPromise
                    .then((fp) => fp.get())
                    .then((result) => {
                    if (result.sealedResult) {
                        this.log.debug(`FP result: ${result.sealedResult}`);
                        resolve({ results: result.sealedResult });
                    }
                    else {
                        const msg = 'Cannot found sealed result in Fingerprint returned payload';
                        this.log.warn(msg);
                        resolve({ error: msg });
                    }
                })
                    .catch((error) => {
                    const msg = `Error in collecting Fingerprint data: ${error.toString}`;
                    this.log.warn(msg);
                    resolve({ error: msg });
                });
            }
        });
    }
    shouldCollectFP() {
        var _a, _b, _c;
        if ((_c = (_b = (_a = this.claims) === null || _a === void 0 ? void 0 : _a.auth.subs.dev) === null || _b === void 0 ? void 0 : _b.sgnls) === null || _c === void 0 ? void 0 : _c.fpt) {
            return true;
        }
        return false;
    }
    parseJwt(token) {
        return JSON.parse(atob(token.split('.')[1]));
    }
}
exports.default = AuthSession;
