"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const base64_1 = __importDefault(require("../common/base64"));
const logger_1 = require("../common/logger");
const auth_error_1 = __importDefault(require("./auth-error"));
const error_code_1 = __importDefault(require("./error-code"));
const device_passive_verify_step_1 = __importDefault(require("./device-passive-verify-step"));
class DevicePassiveRegisterStep {
    constructor() {
        this.log = logger_1.LoggerFactory.getLogger('device-passive-register-step');
        this.name = DevicePassiveRegisterStep.NAME;
    }
    execute(session) {
        return new Promise((resolve, reject) => {
            if (DevicePassiveRegisterStep.hasAssertionResponse(session)) {
                device_passive_verify_step_1.default.runFidoVerify(this.log, session).then(resolve).catch(reject);
            }
            else {
                session
                    .getDeviceRegistration()
                    .then((devRegistration) => {
                    if (devRegistration) {
                        session.embedFpResultToDeviceRegistration(devRegistration).then((devRegistration) => {
                            this.finishRegistration(session, [this.getFido2Registration(session)], devRegistration.getSignals())
                                .then(resolve)
                                .catch(reject);
                        });
                    }
                    else {
                        session.platform.deviceAuth
                            .createRegistration({
                            namespace: session.namespace,
                            endpoint: session.backendOrigin,
                        })
                            .then((devRegistration) => session.embedFpResultToDeviceRegistration(devRegistration))
                            .then((devRegistration) => {
                            devRegistration
                                .getAuthRegistration(session.challenge)
                                .then((authRegistration) => this.finishRegistration(session, [this.getFido2Registration(session), authRegistration], devRegistration.getSignals()))
                                .then((next) => {
                                devRegistration.deviceId = session.settings.deviceId;
                                session.platform.deviceAuth
                                    .storeRegistration(devRegistration)
                                    .then(() => resolve(next))
                                    .catch(reject);
                            })
                                .catch(reject);
                        })
                            .catch(reject);
                    }
                })
                    .catch(reject);
            }
        });
    }
    static hasAssertionResponse(session) {
        const credential = session.credential;
        const assertion = credential.response;
        return assertion && 'authenticatorData' in assertion && 'signature' in assertion;
    }
    finishRegistration(session, registrations, signals) {
        return new Promise((resolve, reject) => {
            session
                .fetchFromBackend('/v1/client/device/fido2/register/finish', {
                deviceName: session.platform.getPlatformName(),
                deviceCapabilities: session.platform.getDeviceCapabilities(),
                registrations: registrations,
                signals: signals,
            })
                .then((response) => {
                if (response.error) {
                    reject(new auth_error_1.default(response.error.message, response.error.code, response.next, false));
                }
                else {
                    const data = response.data;
                    if (data) {
                        if (data.deviceId) {
                            session.settings.deviceId = data.deviceId;
                            session.settings.fidoPasskeyRegistered = true;
                        }
                        if (data.scanMessage) {
                            session.authMessage = data.scanMessage;
                        }
                        resolve(response.next);
                    }
                    else {
                        reject(new auth_error_1.default('Failed to register device, returned deviceId is null or empty', error_code_1.default.ERROR_AUTHENTICATION_FAILURE, response.next, false));
                    }
                }
            })
                .catch(reject);
        });
    }
    getFido2Registration(session) {
        const credential = session.credential;
        const attestation = credential.response;
        return {
            webAuthnCredential: {
                type: credential === null || credential === void 0 ? void 0 : credential.type,
                id: credential === null || credential === void 0 ? void 0 : credential.id,
                rawId: base64_1.default.bufferEncode(credential === null || credential === void 0 ? void 0 : credential.rawId),
                response: {
                    attestationObject: base64_1.default.bufferEncode(attestation.attestationObject),
                    clientDataJSON: base64_1.default.bufferEncode(attestation.clientDataJSON),
                },
            },
        };
    }
}
DevicePassiveRegisterStep.NAME = 'device/passive/register';
exports.default = DevicePassiveRegisterStep;
