"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const logger_1 = require("../common/logger");
const auth_error_1 = __importDefault(require("./auth-error"));
class DevicePassiveSilentStep {
    constructor(forUPK, userConsentStep) {
        this.name = DevicePassiveSilentStep.NAME;
        this.log = logger_1.LoggerFactory.getLogger('device-passive-silent-step');
        this.forUPK = forUPK;
        this.userConsentStep = userConsentStep !== null && userConsentStep !== void 0 ? userConsentStep : {
            execute: () => Promise.resolve({ consentGranted: true }),
        };
    }
    execute(session) {
        this.log.trace('Executing');
        return new Promise((resolve, reject) => {
            session
                .getDeviceRegistration()
                .then((registration) => {
                if (registration) {
                    session.embedFpResultToDeviceRegistration(registration).then((registration) => {
                        this.verify(session, registration).then(resolve).catch(reject);
                    });
                }
                else {
                    this.userConsentStep
                        .execute()
                        .then((output) => {
                        if (output.consentGranted) {
                            session.platform.deviceAuth
                                .createRegistration({
                                namespace: session.namespace,
                                endpoint: session.backendOrigin,
                            })
                                .then((registration) => session.embedFpResultToDeviceRegistration(registration))
                                .then((registration) => this.register(session, registration))
                                .then(resolve)
                                .catch(reject);
                        }
                        else {
                            reject(new Error('User denied the consent to register the device'));
                        }
                    })
                        .catch(reject);
                }
            })
                .catch(reject);
        });
    }
    getBackendRegisterEndpoint() {
        return this.forUPK
            ? '/v1/client/device/universal/register'
            : '/v1/client/device/passive/register';
    }
    getBackendVerifyEndpoint() {
        return this.forUPK ? '/v1/client/device/universal/verify' : '/v1/client/device/passive/verify';
    }
    register(session, registration) {
        this.log.trace('Registering');
        return new Promise((resolve, reject) => {
            registration.getAuthRegistration(session.challenge).then((authRegistration) => {
                session
                    .fetchFromBackend(this.getBackendRegisterEndpoint(), {
                    deviceName: session.platform.getPlatformName(),
                    deviceCapabilities: session.platform.getDeviceCapabilities(),
                    registrations: [authRegistration],
                    signals: registration.getSignals(),
                })
                    .then((response) => {
                    if (response.error) {
                        reject(new auth_error_1.default(response.error.message, response.error.code, response.next, false));
                    }
                    else {
                        const deviceRegisterAuthResp = response;
                        const deviceId = deviceRegisterAuthResp.data.deviceId;
                        if (!deviceId) {
                            reject(new auth_error_1.default('Failed to register device, returned deviceId is null or empty', 0, response.next));
                        }
                        session.settings.deviceId = deviceId;
                        if (deviceRegisterAuthResp.data.passkey != null) {
                            session.settings.fidoPasskeyRegistered = deviceRegisterAuthResp.data.passkey;
                        }
                        registration.deviceId = deviceId;
                        this.log.debug('Device ID: ' + deviceId);
                        const data = response.data;
                        if (data && data.scanMessage) {
                            session.authMessage = data.scanMessage;
                        }
                        session.platform.deviceAuth
                            .storeRegistration(registration)
                            .then(() => resolve(response.next))
                            .catch(reject);
                    }
                })
                    .catch(reject);
            });
        });
    }
    verify(session, registration) {
        this.log.trace('Verifying');
        return new Promise((resolve, reject) => {
            if (registration.deviceId) {
                const challenge = registration.deviceId + ':' + session.challenge;
                registration.sign(challenge).then((signature) => {
                    session
                        .fetchFromBackend(this.getBackendVerifyEndpoint(), {
                        deviceId: registration.deviceId,
                        keyId: registration.keyId,
                        signature: signature,
                        signals: registration.getSignals(),
                    })
                        .then((response) => {
                        if (response.error) {
                            reject(new auth_error_1.default(response.error.message, response.error.code, response.next, false));
                        }
                        else {
                            resolve(response.next);
                        }
                    })
                        .catch(reject);
                });
            }
            else {
                reject(new auth_error_1.default('Failed to initiate verification, DeviceId is missing'));
            }
        });
    }
}
DevicePassiveSilentStep.NAME = 'device/passive/silent';
exports.default = DevicePassiveSilentStep;
