"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DevicePassiveActions = void 0;
const logger_1 = require("../common/logger");
const base64_1 = __importDefault(require("../common/base64"));
const device_passive_silent_step_1 = __importDefault(require("./device-passive-silent-step"));
const auth_token_claims_1 = require("./auth-token-claims");
const auth_error_1 = __importDefault(require("./auth-error"));
const authenticator_builder_1 = require("../authenticator-builder");
const auth_status_actions_1 = require("./auth-status-actions");
const fido_options_error_1 = require("./fido-options-error");
const auth_response_status_1 = require("./auth-response-status");
class DevicePassiveActions extends auth_status_actions_1.AuthStatusActions {
    constructor(getDisplayName, handler) {
        super();
        this.log = logger_1.LoggerFactory.getLogger('device-passive-actions');
        this.getDisplayName = getDisplayName ? getDisplayName : () => null;
        this.handler = handler;
    }
    register(session, signals) {
        this.log.trace('Registering Passkey');
        return new Promise((resolve, reject) => {
            const displayName = this.getDisplayName();
            session
                .fetchFromBackend('/v1/client/device/fido2/register/start', {
                displayName: displayName ? displayName : undefined,
                signals: signals,
            })
                .then((response) => {
                var _a, _b;
                if (response.error) {
                    reject(new auth_error_1.default(response.error.message, response.error.code, response.next, false));
                }
                else {
                    const creationOptions = (_a = response.data) === null || _a === void 0 ? void 0 : _a.credCreateOptions;
                    const requestOptions = (_b = response.data) === null || _b === void 0 ? void 0 : _b.credRequestOptions;
                    if (creationOptions) {
                        this.createCredentials(session, displayName, creationOptions, response)
                            .then(resolve)
                            .catch(reject);
                    }
                    else if (requestOptions) {
                        this.getCredentials(session, requestOptions, response).then(resolve).catch(reject);
                    }
                    else {
                        const error = new auth_error_1.default(DevicePassiveActions.NO_CREDS_FOUND);
                        reject(error);
                    }
                }
            })
                .catch(reject);
        });
    }
    verify(session) {
        this.log.trace('Verifying Passkey');
        return new Promise((resolve, reject) => {
            if (session.settings.deviceId) {
                session
                    .fetchFromBackend('/v1/client/device/fido2/verify/start', {
                    deviceId: session.settings.deviceId,
                })
                    .then((response) => {
                    if (response.error) {
                        reject(new auth_error_1.default(response.error.message, response.error.code, response.next, false));
                    }
                    else {
                        const options = response.data.credRequestOptions;
                        this.getCredentials(session, options, response).then(resolve).catch(reject);
                    }
                })
                    .catch(reject);
            }
            else {
                reject(new auth_error_1.default('Failed to start verification, DeviceId is missing'));
            }
        });
    }
    createCredentials(session, displayName, options, response) {
        this.log.trace('Trying create new FIDO credentials');
        return new Promise((resolve, reject) => {
            options.challenge = base64_1.default.bufferDecode(options.challenge);
            options.user.id = base64_1.default.bufferDecode(options.user.id);
            if (displayName) {
                options.user.displayName = displayName;
            }
            if (options.excludeCredentials) {
                options.excludeCredentials.forEach((i) => {
                    i.id = base64_1.default.bufferDecode(i.id);
                });
            }
            const fidoCreationOptions = {
                publicKey: options,
            };
            session.platform.webauthn
                .createCredentials(fidoCreationOptions)
                .then((credential) => {
                if (!credential) {
                    reject(new auth_error_1.default('Failed to create FIDO2 credentials'));
                }
                else {
                    session.credential = credential;
                    resolve(response.next);
                }
            })
                .catch((error) => {
                const fidoCreationError = fido_options_error_1.FidoOptionsError.identifyRegistrationError({
                    error: error,
                    options: fidoCreationOptions,
                    platform: session.platform,
                });
                if (fidoCreationError instanceof fido_options_error_1.FidoOptionsError &&
                    fidoCreationError.name === 'InvalidStateError') {
                    if (this.handler) {
                        this.log.trace('Passkey handler callback has been defined');
                        this.handler()
                            .then((userResponse) => {
                            if (userResponse === auth_response_status_1.AuthResponseStatus.Accept) {
                                this.parseCredRequestOptions(response, session).then(resolve).catch(reject);
                            }
                            else {
                                reject(new auth_error_1.default(`${DevicePassiveActions.USER_NOT_ACCEPTING_RESPONSE}: ${userResponse}`));
                                return;
                            }
                        })
                            .catch(reject);
                    }
                    else {
                        this.log.trace('Passkey handler callback is null, go ahead with passkey discoverable as default behavior');
                        this.parseCredRequestOptions(response, session).then(resolve).catch(reject);
                    }
                }
                else {
                    reject(fidoCreationError);
                }
            });
        });
    }
    parseCredRequestOptions(response, session) {
        return new Promise((resolve, reject) => {
            const data = response.data;
            if (data.credRequestOptions) {
                const requestOptions = data.credRequestOptions;
                this.getCredentials(session, requestOptions, response).then(resolve).catch(reject);
            }
            else {
                reject(new auth_error_1.default(DevicePassiveActions.NO_REQUEST_CREDS_FOUND));
            }
        });
    }
    getCredentials(session, options, response) {
        this.log.trace('Trying get existing FIDO credentials');
        return new Promise((resolve, reject) => {
            var _a, _b;
            if (((_a = options.allowCredentials) === null || _a === void 0 ? void 0 : _a.length) == 0) {
                reject(new auth_error_1.default('AllowCredentials array should not be empty when call getCredentials()'));
            }
            else {
                options.challenge = base64_1.default.bufferDecode(options.challenge);
                if (options.allowCredentials) {
                    (_b = options.allowCredentials) === null || _b === void 0 ? void 0 : _b.forEach((i) => {
                        i.id = base64_1.default.bufferDecode(i.id);
                    });
                }
                const fidoRequestOptions = {
                    publicKey: options,
                };
                session.platform.webauthn
                    .getCredentials(fidoRequestOptions)
                    .then((credential) => {
                    if (!credential) {
                        reject(new auth_error_1.default('Failed to load FIDO2 credentials'));
                    }
                    else {
                        session.credential = credential;
                        resolve(response.next);
                    }
                })
                    .catch((error) => {
                    const fidoAuthenticationError = fido_options_error_1.FidoOptionsError.identifyAuthenticationError({
                        error: error,
                        options: fidoRequestOptions,
                        platform: session.platform,
                    });
                    reject(fidoAuthenticationError);
                });
            }
        });
    }
}
DevicePassiveActions.NO_REQUEST_CREDS_FOUND = 'Passkey has already been registered but found no CredentialRequestOptions in the fido/register/start response payload';
DevicePassiveActions.NO_CREDS_FOUND = 'Neither credCreateOptions nor credRequestOptions are found in the fido/register/start response payload';
DevicePassiveActions.USER_NOT_ACCEPTING_RESPONSE = 'User not accepting to continue by reusing the existing passkey with user response';
exports.DevicePassiveActions = DevicePassiveActions;
class DevicePassiveStep extends DevicePassiveActions {
    constructor(getDisplayName, handler, role) {
        super(getDisplayName, handler);
        this.name = DevicePassiveStep.NAME;
        this.role = role !== null && role !== void 0 ? role : authenticator_builder_1.DeviceRole.Primary;
        this.log = logger_1.LoggerFactory.getLogger('device-passive-step');
    }
    execute(session) {
        if (this.role == authenticator_builder_1.DeviceRole.Secondary) {
            return this.waitForStatus(session);
        }
        if (!session.platform.isFidoSupported() ||
            session.uvLevel === auth_token_claims_1.UserVerificationLevel.Discouraged) {
            return Promise.resolve(device_passive_silent_step_1.default.NAME);
        }
        if (session.settings.deviceId) {
            if (session.settings.fidoPasskeyRegistered) {
                return this.verify(session);
            }
            return Promise.resolve(device_passive_silent_step_1.default.NAME);
        }
        return new Promise((resolve, reject) => {
            session
                .getFingerprintData()
                .then((signal) => {
                const signals = {
                    fingerprint: signal,
                };
                this.register(session, signals).then(resolve).catch(reject);
            })
                .catch((error) => {
                const errorMsg = `Unexpected error happened during Fingerprint data collection: ${error.message}`;
                this.log.warn(errorMsg);
                this.log.warn(error);
                const signals = {
                    fingerprint: {
                        error: errorMsg,
                    },
                };
                this.register(session, signals).then(resolve).catch(reject);
            });
        });
    }
}
DevicePassiveStep.NAME = 'device/passive';
exports.default = DevicePassiveStep;
