"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const base64_1 = __importDefault(require("../common/base64"));
const logger_1 = require("../common/logger");
const auth_error_1 = __importDefault(require("./auth-error"));
class DevicePassiveVerifyStep {
    constructor() {
        this.log = logger_1.LoggerFactory.getLogger('device-passive-verify-step');
        this.name = DevicePassiveVerifyStep.NAME;
    }
    execute(session) {
        return new Promise((resolve, reject) => {
            DevicePassiveVerifyStep.runFidoVerify(this.log, session).then(resolve).catch(reject);
        });
    }
    static runFidoVerify(log, session) {
        return new Promise((resolve, reject) => {
            session
                .getFingerprintData()
                .then((signal) => {
                const signals = { fingerprint: signal };
                DevicePassiveVerifyStep.makeFidoVerifyFinishRequest(log, session, signals)
                    .then(resolve)
                    .catch(reject);
            })
                .catch((error) => {
                const errorMsg = `Unexpected error happened during Fingerprint data collection ${error.toString}`;
                log.warn(errorMsg);
                const signals = { fingerprint: { error: errorMsg } };
                DevicePassiveVerifyStep.makeFidoVerifyFinishRequest(log, session, signals)
                    .then(resolve)
                    .catch(reject);
            });
        });
    }
    static makeFidoVerifyFinishRequest(log, session, signals) {
        return new Promise((resolve, reject) => {
            const credential = session.credential;
            const assertion = credential.response;
            session
                .fetchFromBackend('/v1/client/device/fido2/verify/finish', {
                webAuthnAssertion: {
                    type: credential === null || credential === void 0 ? void 0 : credential.type,
                    id: credential === null || credential === void 0 ? void 0 : credential.id,
                    rawId: base64_1.default.bufferEncode(credential === null || credential === void 0 ? void 0 : credential.rawId),
                    response: {
                        authenticatorData: base64_1.default.bufferEncode(assertion.authenticatorData),
                        clientDataJSON: base64_1.default.bufferEncode(assertion.clientDataJSON),
                        signature: base64_1.default.bufferEncode(assertion.signature),
                        userHandle: assertion.userHandle
                            ? base64_1.default.bufferEncode(assertion.userHandle)
                            : undefined,
                    },
                },
                signals: signals,
            })
                .then((response) => {
                if (response.error) {
                    reject(new auth_error_1.default(response.error.message, response.error.code, response.next, false));
                }
                else {
                    const data = response.data;
                    if (data) {
                        if (data.deviceId) {
                            session.settings.deviceId = data.deviceId;
                        }
                        if (data.passkey) {
                            session.settings.fidoPasskeyRegistered = true;
                        }
                        if (data.scanMessage) {
                            session.authMessage = data.scanMessage;
                        }
                    }
                    else {
                        log.warn('No data was received in the response');
                    }
                    session.settings.fidoPasskeyRegistered = true;
                    resolve(response.next);
                }
            })
                .catch(reject);
        });
    }
}
DevicePassiveVerifyStep.NAME = 'device/passive/verify';
exports.default = DevicePassiveVerifyStep;
