"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FidoOptionsError = void 0;
class FidoOptionsError extends Error {
    constructor({ message, cause, name }) {
        super(message, { cause });
        this.name = name !== null && name !== void 0 ? name : cause.name;
    }
    static identifyAuthenticationError({ error, options, platform, }) {
        const { publicKey } = options;
        if (!publicKey) {
            return new FidoOptionsError({
                message: FidoOptionsError.MISSING_PUBLIC_KEY_PROPERTY,
                cause: new Error(FidoOptionsError.MISSING_PUBLIC_KEY_PROPERTY),
            });
        }
        if (error.name === 'AbortError') {
            if (options.signal instanceof AbortSignal) {
                return new FidoOptionsError({
                    message: FidoOptionsError.ABORT_SIGNAL,
                    cause: error,
                });
            }
        }
        else if (error.name === 'NotAllowedError') {
            return new FidoOptionsError({
                message: error.message,
                cause: error,
            });
        }
        else if (error.name === 'SecurityError') {
            const currentDomain = platform.getOrigin();
            if (publicKey.rpId !== currentDomain) {
                return new FidoOptionsError({
                    message: `${FidoOptionsError.INVALID_RP_ID}: ${publicKey.rpId}`,
                    cause: error,
                });
            }
            else {
                return new FidoOptionsError({
                    message: `${FidoOptionsError.SECURITY_ERROR_FOUND}: ${error.message}`,
                    cause: error,
                });
            }
        }
        else if (error.name === 'UnknownError') {
            return new FidoOptionsError({
                message: FidoOptionsError.UNKNOWN_ERROR,
                cause: error,
            });
        }
        return error;
    }
    static identifyRegistrationError({ error, options, platform, }) {
        var _a, _b, _c;
        const { publicKey } = options;
        if (!publicKey) {
            return new FidoOptionsError({
                message: FidoOptionsError.MISSING_PUBLIC_KEY_PROPERTY,
                cause: new Error(FidoOptionsError.MISSING_PUBLIC_KEY_PROPERTY),
            });
        }
        if (error.name === 'InvalidStateError') {
            return new FidoOptionsError({
                message: FidoOptionsError.INVALID_STATE_ERROR,
                cause: error,
            });
        }
        else if (error.name === 'AbortError') {
            if (options.signal instanceof AbortSignal) {
                return new FidoOptionsError({
                    message: this.ABORT_SIGNAL,
                    cause: error,
                });
            }
        }
        else if (error.name === 'ConstraintError') {
            if (((_a = publicKey.authenticatorSelection) === null || _a === void 0 ? void 0 : _a.requireResidentKey) === true) {
                return new FidoOptionsError({
                    message: this.CONSTRAINT_ERROR,
                    cause: error,
                });
            }
            else if (options.mediation === 'conditional' &&
                ((_b = publicKey.authenticatorSelection) === null || _b === void 0 ? void 0 : _b.userVerification) === 'required') {
                return new FidoOptionsError({
                    message: this.USER_VERIFICATION_NOT_POSSILE,
                    cause: error,
                });
            }
            else if (((_c = publicKey.authenticatorSelection) === null || _c === void 0 ? void 0 : _c.userVerification) === 'required') {
                return new FidoOptionsError({
                    message: this.NO_MATCHING_AUTHENTICATOR,
                    cause: error,
                });
            }
        }
        else if (error.name === 'NotAllowedError') {
            return new FidoOptionsError({
                message: error.message,
                cause: error,
            });
        }
        else if (error.name === 'NotSupportedError') {
            const validPubKeyCredParams = publicKey.pubKeyCredParams.filter((param) => param.type === 'public-key');
            if (validPubKeyCredParams.length === 0) {
                return new FidoOptionsError({
                    message: this.INVALID_CRED_PARAMS,
                    cause: error,
                });
            }
            return new FidoOptionsError({
                message: FidoOptionsError.NO_MATCHING_AUTHENTICATOR_FOR_PARAMS_ALGO,
                cause: error,
            });
        }
        else if (error.name === 'SecurityError') {
            const currentDomain = platform.getOrigin();
            if (publicKey.rp.id !== currentDomain) {
                return new FidoOptionsError({
                    message: `${FidoOptionsError.INVALID_RP_ID}: ${publicKey.rp.id}`,
                    cause: error,
                });
            }
            else {
                return new FidoOptionsError({
                    message: `${FidoOptionsError.SECURITY_ERROR_FOUND}: ${error.message}`,
                    cause: error,
                });
            }
        }
        else if (error.name === 'TypeError') {
            if (publicKey.user.id.byteLength < 1 || publicKey.user.id.byteLength > 64) {
                return new FidoOptionsError({
                    message: FidoOptionsError.INVALID_USER_ID_LENGTH,
                    cause: error,
                });
            }
        }
        else if (error.name === 'UnknownError') {
            return new FidoOptionsError({
                message: FidoOptionsError.UNKNOWN_ERROR,
                cause: error,
            });
        }
        return error;
    }
}
FidoOptionsError.MISSING_PUBLIC_KEY_PROPERTY = 'options missing publicKey property';
FidoOptionsError.ABORT_SIGNAL = 'Authentication was sent an abort signal';
FidoOptionsError.INVALID_STATE_ERROR = 'The authenticator was already registered';
FidoOptionsError.UNKNOWN_ERROR = 'Unknown error: the authenticator was unable to process the predefined options, or unable to generate a new credential';
FidoOptionsError.SECURITY_ERROR_FOUND = 'Security error found';
FidoOptionsError.INVALID_RP_ID = 'The rp.id is not valid for the current domain';
FidoOptionsError.CONSTRAINT_ERROR = 'Discoverable credentials is required but found no matching supported authenticator';
FidoOptionsError.USER_VERIFICATION_NOT_POSSILE = 'User verification is required during automatic registration but this could not be performed';
FidoOptionsError.NO_MATCHING_AUTHENTICATOR = 'User verification is required but found no matching supported authenticator';
FidoOptionsError.INVALID_CRED_PARAMS = 'No entry in pubKeyCredParams having the type of public-key';
FidoOptionsError.NO_MATCHING_AUTHENTICATOR_FOR_PARAMS_ALGO = 'No available authenticator supported any of the specified pubKeyCredParams algorithms';
FidoOptionsError.INVALID_USER_ID_LENGTH = 'User ID has invalid length';
exports.FidoOptionsError = FidoOptionsError;
