"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const logger_1 = require("../common/logger");
const cancelable_promise_1 = __importDefault(require("../common/cancelable-promise"));
const auth_session_1 = __importDefault(require("./auth-session"));
const auth_error_1 = __importDefault(require("./auth-error"));
const report_error_step_1 = __importDefault(require("./report-error-step"));
const error_code_1 = __importDefault(require("./error-code"));
class MainAuthenticator {
    constructor(platform, settings, finishStep, steps) {
        this.steps = new Map();
        this.log = logger_1.LoggerFactory.getLogger('main-authenticator');
        this.platform = platform;
        this.authFinishStep = finishStep;
        this.settings = settings;
        if (steps) {
            for (let step of steps) {
                this.steps.set(step.name, step);
            }
        }
    }
    isPasskeyRegistered() {
        return this.settings.fidoPasskeyRegistered;
    }
    isFidoSupported() {
        return this.platform.isFidoSupported();
    }
    isDeviceRegistered() {
        return this.settings.deviceId !== null;
    }
    isMobileWeb() {
        const userAgent = this.platform.getUserAgent();
        return (userAgent !== null && /Mobi|Android|webOS|iPhone|iPad|BlackBerry|Opera Mini/i.test(userAgent));
    }
    getDeviceId() {
        return this.settings.deviceId;
    }
    resetDeviceSettings() {
        this.settings.reset();
        this.platform.deviceAuth.reset();
    }
    authenticate(authToken) {
        return new cancelable_promise_1.default((resolve, reject, onCancel) => {
            if (!authToken) {
                reject(new Error('No authentication token provided'));
            }
            if (!this.authFinishStep) {
                reject(new Error('AuthFinish step must be specified'));
            }
            try {
                const session = new auth_session_1.default(this.settings, this.platform, authToken);
                const processing = this.process(session);
                onCancel(() => processing.cancel());
                processing
                    .then(() => {
                    var _a;
                    if (session.lastStep !== MainAuthenticator.AUTH_EMPTY) {
                        this.log.info('Authentication flow has been completed.');
                        return (_a = this.authFinishStep) === null || _a === void 0 ? void 0 : _a.execute({ authId: session.authId });
                    }
                    else {
                        this.log.info('Next step is not provided, authentication flow is terminated without completion.');
                        resolve();
                    }
                })
                    .then(resolve)
                    .catch(reject);
            }
            catch (e) {
                reject(e);
            }
        });
    }
    unregisterDevice() {
        if (!this.isDeviceRegistered()) {
            return Promise.resolve();
        }
        return new Promise((resolve, reject) => {
            this.unregister('/v1/client/device/unregister')
                .then(() => {
                this.resetDeviceSettings();
                resolve();
            })
                .catch(reject);
        });
    }
    unregisterPasskey() {
        if (!this.settings.fidoPasskeyRegistered) {
            return Promise.resolve();
        }
        return new Promise((resolve, reject) => {
            this.unregister('/v1/client/stepup/unregister')
                .then(() => {
                this.settings.fidoPasskeyRegistered = false;
                resolve();
            })
                .catch(reject);
        });
    }
    unregister(endpointPath) {
        return new Promise((resolve, reject) => {
            try {
                let session = new auth_session_1.default(this.settings, this.platform);
                session
                    .fetchFromBackend(endpointPath, {
                    deviceId: this.getDeviceId(),
                })
                    .then((response) => {
                    if (response.error) {
                        reject(new auth_error_1.default(response.error.message, response.error.code, response.next, false));
                    }
                    else {
                        resolve();
                    }
                })
                    .catch(reject);
            }
            catch (e) {
                reject(e);
            }
        });
    }
    process(session) {
        return new cancelable_promise_1.default((resolve, reject, onCancel) => {
            onCancel(() => {
                this.log.info('Canceled');
                session.closeAllMessageChannels();
                resolve();
            });
            this.nextStep(session, session.next, 1).then(resolve).catch(reject);
        });
    }
    nextStep(session, step, attempt) {
        this.log.debug(`Authentication attempt ${attempt}, next step: ${step}`);
        session.lastStep = step;
        return new Promise((resolve, reject) => {
            if ([MainAuthenticator.AUTH_DONE, MainAuthenticator.AUTH_EMPTY].includes(step)) {
                resolve();
            }
            else if (attempt > MainAuthenticator.MAX_ATTEMPTS) {
                reject(new auth_error_1.default('Too many authentication steps', error_code_1.default.ERROR_MAX_ATTEMPTS));
            }
            else {
                this.getNextStep(step)
                    .execute(session)
                    .then((next) => this.nextStep(session, next, attempt + 1))
                    .then(resolve)
                    .catch((e) => new report_error_step_1.default(e)
                    .execute(session)
                    .then((next) => this.nextStep(session, next, attempt + 1))
                    .then(resolve)
                    .catch(reject));
            }
        });
    }
    getNextStep(step) {
        var nextStep = this.steps.get(step);
        if (nextStep) {
            return nextStep;
        }
        return new report_error_step_1.default('Unknown authentication step: ' + step);
    }
}
MainAuthenticator.AUTH_DONE = 'done';
MainAuthenticator.AUTH_EMPTY = '';
MainAuthenticator.MAX_ATTEMPTS = 50;
exports.default = MainAuthenticator;
