"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const auth_error_1 = __importDefault(require("./auth-error"));
const logger_1 = require("../common/logger");
const authenticator_builder_1 = require("../authenticator-builder");
const error_code_1 = __importDefault(require("./error-code"));
class MobileInstantStep {
    constructor(implementation, getDeviceIp) {
        this.name = MobileInstantStep.NAME;
        this.log = logger_1.LoggerFactory.getLogger('mobile-instant-step');
        this.errorCodeBak = error_code_1.default.ERROR_AUTHENTICATION_CLIENT_FAILURE;
        this.implementation = implementation;
        if (getDeviceIp) {
            this.getDeviceIp = getDeviceIp;
            this.deviceIpDetection = false;
        }
        else {
            this.getDeviceIp = () => null;
            this.deviceIpDetection = true;
        }
    }
    execute(session) {
        return new Promise((resolve, reject) => {
            let builder = session.platform
                .getMobileAuthBuilder()
                .withDeviceIpDetection(this.deviceIpDetection)
                .withIpDiscoveryUrl('https://device.payfone.com/whatismyipaddress');
            switch (this.implementation) {
                case authenticator_builder_1.MobileAuthImplementation.Pixel: {
                    builder = builder.withPixelImplementation();
                    break;
                }
                case authenticator_builder_1.MobileAuthImplementation.Fetch: {
                    builder = builder.withFetchImplementation().withFinishStep((input) => {
                        return new Promise((resolve, reject) => {
                            session
                                .fetchFromBackend('/v1/client/mobile/instant/finish', {
                                vfp: input.vfp,
                            })
                                .then(resolve)
                                .catch(reject);
                        });
                    });
                    break;
                }
            }
            builder = builder.withStartStep((input) => {
                return new Promise((resolve, reject) => {
                    const ip = this.deviceIpDetection ? input.deviceDescriptor.ip : this.getDeviceIp();
                    session
                        .fetchFromBackend('/v1/client/mobile/instant/start', {
                        cellularIp: ip,
                        implementation: this.implementation,
                    })
                        .then((response) => {
                        var _a;
                        if (response.error) {
                            this.nextBak = response.next;
                            this.errorCodeBak = response.error.code;
                            reject(new auth_error_1.default(response.error.message, response.error.code, response.next, false));
                        }
                        else {
                            resolve({
                                authUrl: (_a = response.data) === null || _a === void 0 ? void 0 : _a.redirectUrl,
                            });
                        }
                    })
                        .catch(reject);
                });
            });
            const authenticator = builder.build();
            authenticator
                .authenticate()
                .then((response) => {
                if (response) {
                    const authResponse = response;
                    if (authResponse.error) {
                        reject(new auth_error_1.default(response.error.message, response.error.code, response.next, false));
                    }
                    resolve(authResponse.next);
                }
                else {
                    session
                        .fetchFromBackend('/v1/client/mobile/instant/finish', {})
                        .then((response) => resolve(response.next))
                        .catch((error) => {
                        reject(new auth_error_1.default(response.error.message, response.error.code, response.next, false));
                    });
                }
            })
                .catch((error) => {
                reject(new auth_error_1.default(error, this.errorCodeBak, this.nextBak, false));
            });
        });
    }
}
MobileInstantStep.NAME = 'mobile/instant';
exports.default = MobileInstantStep;
