"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const logger_1 = require("../common/logger");
const instantlink_1 = require("../instantlink");
const phone_number_input_1 = require("./phone-number-input");
const auth_error_1 = __importDefault(require("./auth-error"));
const auth_status_actions_1 = require("./auth-status-actions");
const mobile_otp_step_1 = __importDefault(require("./mobile-otp-step"));
const SIMULATED_LINK_CLICK_DELAY = 100;
class MobileInstantLinkStep extends auth_status_actions_1.AuthStatusActions {
    constructor(startStep, retryStep, getDeviceIp) {
        super();
        this.name = MobileInstantLinkStep.NAME;
        this.log = logger_1.LoggerFactory.getLogger('mobile-instantlink-step');
        this.startStep = startStep;
        this.retryStep = retryStep;
        this.getDeviceIp = getDeviceIp !== null && getDeviceIp !== void 0 ? getDeviceIp : (() => null);
    }
    isTestMode(session) {
        var _a, _b, _c;
        var testMode = false;
        if ((_c = (_b = (_a = session.claims) === null || _a === void 0 ? void 0 : _a.auth.subs.mob) === null || _b === void 0 ? void 0 : _b.auths.inln) === null || _c === void 0 ? void 0 : _c.tme) {
            testMode = true;
        }
        return testMode;
    }
    execute(session) {
        this.log.trace('Executing');
        return new Promise((resolve, reject) => {
            var _a, _b, _c;
            var phoneNumberNeeded = true;
            if ((_c = (_b = (_a = session.claims) === null || _a === void 0 ? void 0 : _a.auth.subs.mob) === null || _b === void 0 ? void 0 : _b.auths.inln) === null || _c === void 0 ? void 0 : _c.mnp) {
                phoneNumberNeeded = false;
            }
            this.runStartStep(session, phoneNumberNeeded)
                .catch(reject);
            this.waitForStatus(session).then(resolve).catch(reject);
        });
    }
    getInstantLinkError(responseData) {
        const errorCode = responseData.code;
        var errorMessage = '';
        if (errorCode) {
            errorMessage += `Error Code: ${responseData.code}, `;
        }
        let instantLinkError;
        if (errorCode === 10005) {
            if (responseData.message) {
                errorMessage += `${responseData.message}`;
            }
            else {
                errorMessage += `Max number of retires reached.`;
            }
            instantLinkError = new instantlink_1.InstantLinkMaxRetryError(errorMessage, responseData.code);
        }
        else {
            if (responseData.message) {
                errorMessage += `${responseData.message}`;
            }
            else {
                errorMessage += `Error validating phone number`;
            }
            instantLinkError = new phone_number_input_1.PhoneValidationError(errorMessage, responseData.code);
        }
        this.log.error(`Server reports instant link error: ${errorMessage}`);
        return instantLinkError;
    }
    runStartStep(session, phoneNumberNeeded, instantLinkError) {
        return new Promise((resolve, reject) => {
            if (this.startStep) {
                this.startStep
                    .execute(phoneNumberNeeded, instantLinkError)
                    .then((input) => {
                    const inputError = mobile_otp_step_1.default.validatePhoneNumberInput(input);
                    if (inputError) {
                        reject(inputError);
                    }
                    else {
                        session
                            .fetchFromBackend('/v1/client/mobile/instantlink/start', {
                            sourceIp: this.getDeviceIp(),
                            mobileNumber: input === null || input === void 0 ? void 0 : input.phoneNumber,
                        })
                            .then((response) => {
                            const authResponse = response;
                            if (authResponse.error) {
                                reject(new auth_error_1.default(authResponse.error.message, authResponse.error.code, response.next, false));
                            }
                            else if (authResponse.data) {
                                const errorCode = authResponse.data.code;
                                var errorMessage = '';
                                if (errorCode) {
                                    errorMessage += `Error Code: ${authResponse.data.code}, `;
                                }
                                if (errorCode === 10005) {
                                    if (authResponse.data.message) {
                                        errorMessage += `${authResponse.data.message}`;
                                    }
                                    else {
                                        errorMessage += `Max number of retires reached.`;
                                    }
                                    let maxRetryError = new instantlink_1.InstantLinkMaxRetryError(errorMessage, authResponse.data.code);
                                    if (this.retryStep) {
                                        this.runRetryStep(session, maxRetryError).then(resolve).catch(reject);
                                    }
                                    else {
                                        this.log.error('Max number of retires reached but retry step is undefined.');
                                    }
                                }
                                else {
                                    if (authResponse.data.message) {
                                        errorMessage += `${authResponse.data.message}`;
                                    }
                                    else {
                                        errorMessage += `Error validating phone number.`;
                                    }
                                    let invalidPhoneError = new phone_number_input_1.PhoneValidationError(errorMessage, authResponse.data.code);
                                    this.runStartStep(session, phoneNumberNeeded, invalidPhoneError)
                                        .then(resolve)
                                        .catch(reject);
                                }
                            }
                            else {
                                this.handleTestMode(session)
                                    .then(() => {
                                    if (this.retryStep) {
                                        this.runRetryStep(session).then(resolve).catch(reject);
                                    }
                                    else {
                                        resolve(authResponse.next);
                                    }
                                })
                                    .catch(reject);
                            }
                        })
                            .catch(reject);
                    }
                })
                    .catch(reject);
            }
            else {
                reject(new Error('InstantLink start step must be specified'));
            }
        });
    }
    runRetryStep(session, instantLinkError) {
        return new Promise((resolve, reject) => {
            if (this.retryStep) {
                this.retryStep
                    .execute(instantLinkError)
                    .then((resultType) => {
                    if (resultType === instantlink_1.InstantLinkResultType.OnResend) {
                        session
                            .fetchFromBackend('/v1/client/mobile/instantlink/start', {
                            sourceIp: this.getDeviceIp(),
                            mobileNumber: '',
                        })
                            .then((response) => {
                            const authResponse = response;
                            if (authResponse.error) {
                                reject(new auth_error_1.default(authResponse.error.message, authResponse.error.code, response.next, false));
                            }
                            else if (authResponse.data) {
                                const errorCode = authResponse.data.code;
                                var errorMessage = '';
                                if (errorCode) {
                                    errorMessage += `Error Code: ${authResponse.data.code}, `;
                                }
                                if (authResponse.data.message) {
                                    errorMessage += `${authResponse.data.message}`;
                                }
                                else {
                                    errorMessage += `Max number of retires reached.`;
                                }
                                let maxRetryError = new instantlink_1.InstantLinkMaxRetryError(errorMessage, authResponse.data.code);
                                this.runRetryStep(session, maxRetryError).then(resolve).catch(reject);
                            }
                            else {
                                this.handleTestMode(session)
                                    .then(() => this.runRetryStep(session))
                                    .then(resolve)
                                    .catch(reject);
                            }
                        })
                            .catch(reject);
                    }
                    else if (resultType === instantlink_1.InstantLinkResultType.OnMobileNumberChange) {
                        this.runStartStep(session, true).then(resolve).catch(reject);
                    }
                    else {
                        this.log.warn('Unkown enum of ', resultType);
                    }
                })
                    .catch(reject);
            }
            else {
                this.log.debug("Retry step doesn't exsist, skip.");
                resolve('');
            }
        });
    }
    handleTestMode(session) {
        return new Promise((resolve, reject) => {
            if (this.isTestMode(session)) {
                this.log.info('Simulating user clicking the instant link');
                setTimeout(() => {
                    session.platform
                        .fetch(session.backendOrigin +
                        '/v1/client/mobile/instantlink/finish?token=' +
                        encodeURIComponent(session.authToken) +
                        '&vfp=test-vfp', {
                        mode: 'no-cors',
                        method: 'GET',
                    })
                        .then(() => resolve)
                        .catch((e) => {
                        this.log.error('Calling AuthFinish in test mode has failed: ', auth_error_1.default.extractMessage(e));
                        reject(e);
                    });
                }, SIMULATED_LINK_CLICK_DELAY);
            }
            else {
                resolve();
            }
        });
    }
}
MobileInstantLinkStep.NAME = 'mobile/instantlink';
exports.default = MobileInstantLinkStep;
