"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const logger_1 = require("../common/logger");
const auth_error_1 = __importDefault(require("./auth-error"));
const otp_1 = require("../otp");
const phone_number_input_1 = require("./phone-number-input");
class MobileOtpStep {
    constructor(otpStartStep, otpFinishStep) {
        this.name = MobileOtpStep.NAME;
        this.log = logger_1.LoggerFactory.getLogger('mobile-otp-step');
        this.otpStartStep = otpStartStep;
        this.otpFinishStep = otpFinishStep;
    }
    execute(session) {
        return new Promise((resolve, reject) => {
            var _a, _b, _c;
            if (!this.otpStartStep) {
                reject(new Error('OtpStartStep step must be specified'));
                return;
            }
            if (!this.otpFinishStep) {
                reject(new Error('OtpFinishStep step must be specified'));
                return;
            }
            var phoneNumberNeeded = true;
            if ((_c = (_b = (_a = session.claims) === null || _a === void 0 ? void 0 : _a.auth.subs.mob) === null || _b === void 0 ? void 0 : _b.auths.otp) === null || _c === void 0 ? void 0 : _c.mnp) {
                phoneNumberNeeded = false;
            }
            this.runOtpStartStep(session, this.otpStartStep, this.otpFinishStep, phoneNumberNeeded)
                .then(resolve)
                .catch(reject);
        });
    }
    static validatePhoneNumberInput(input) {
        if (input === null || input === undefined) {
            return;
        }
        if (typeof input !== 'object') {
            return new auth_error_1.default('Input must be an object');
        }
        if (!('phoneNumber' in input)) {
            return new auth_error_1.default('phoneNumber field in the input is missing');
        }
        if (typeof input.phoneNumber !== 'string') {
            return new auth_error_1.default('phoneNumber field in the input must be a string');
        }
        return;
    }
    runOtpStartStep(session, otpStartStep, otpFinishStep, phoneNumberNeeded, phoneValidationError) {
        return new Promise((resolve, reject) => {
            otpStartStep
                .execute(phoneNumberNeeded, phoneValidationError)
                .then((input) => {
                const inputError = MobileOtpStep.validatePhoneNumberInput(input);
                if (inputError) {
                    reject(inputError);
                }
                else {
                    session
                        .fetchFromBackend('/v1/client/mobile/otp/start', {
                        mobileNumber: input === null || input === void 0 ? void 0 : input.phoneNumber,
                    })
                        .then((response) => {
                        const authResponse = response;
                        if (authResponse.error) {
                            reject(new auth_error_1.default(authResponse.error.message, authResponse.error.code, response.next, false));
                        }
                        else if (authResponse.data) {
                            let data = authResponse.data;
                            var errorMessage = '';
                            if (data === null || data === void 0 ? void 0 : data.code) {
                                errorMessage += `Error Code: ${data.code}, `;
                            }
                            if (data === null || data === void 0 ? void 0 : data.message) {
                                errorMessage += `${data.message}`;
                            }
                            else {
                                errorMessage += `Error validating phone number`;
                            }
                            let phoneNumberValidationError = new phone_number_input_1.PhoneValidationError(errorMessage, data === null || data === void 0 ? void 0 : data.code);
                            this.log.error(`Server reports invalid phone number: ${errorMessage}`);
                            this.runOtpStartStep(session, otpStartStep, otpFinishStep, phoneNumberNeeded, phoneNumberValidationError)
                                .then(resolve)
                                .catch(reject);
                        }
                        else {
                            this.runOtpFinishStep(session, otpStartStep, otpFinishStep, phoneNumberNeeded)
                                .then(resolve)
                                .catch(reject);
                        }
                    })
                        .catch(reject);
                }
            })
                .catch((error) => {
                const message = auth_error_1.default.extractMessage(error);
                reject(new Error(`Failed to start OTP flow: ${message}`));
            });
        });
    }
    static validateFinishResult(result) {
        if (result === null || result === undefined || typeof result !== 'object') {
            return new auth_error_1.default('result should be a valid object');
        }
        else if (!('resultType' in result)) {
            return new auth_error_1.default('resultType field in the result is missing');
        }
        else if (result.resultType === otp_1.OtpFinishResultType.OnSuccess) {
            const input = result.input;
            if (input === null || input === undefined || typeof input !== 'object') {
                return new auth_error_1.default('input should be a valid object when resultType field is set with OnSuccess value');
            }
            if (!('otp' in input)) {
                return new auth_error_1.default('otp field in the input is missing');
            }
            if (typeof input.otp !== 'string') {
                return new auth_error_1.default('otp field in the input must be a string');
            }
        }
        return;
    }
    resendOtp(session, otpStartStep, otpFinishStep, phoneNumberNeeded) {
        return new Promise((resolve, reject) => {
            session
                .fetchFromBackend('/v1/client/mobile/otp/start', {})
                .then((response) => {
                const authResponse = response;
                if (authResponse.error) {
                    reject(new auth_error_1.default(authResponse.error.message, authResponse.error.code, response.next, false));
                }
                else {
                    this.runOtpFinishStep(session, otpStartStep, otpFinishStep, phoneNumberNeeded)
                        .then(resolve)
                        .catch(reject);
                }
            })
                .catch(reject);
        });
    }
    runOtpFinishStep(session, otpStartStep, otpFinishStep, phoneNumberNeeded, otpError) {
        return new Promise((resolve, reject) => {
            otpFinishStep
                .execute(otpError)
                .then((otpFinishResult) => {
                var _a;
                let result = otpFinishResult;
                const resultError = MobileOtpStep.validateFinishResult(result);
                if (resultError) {
                    reject(resultError);
                }
                else {
                    switch (result.resultType) {
                        case otp_1.OtpFinishResultType.OnResendOtp:
                            this.resendOtp(session, otpStartStep, otpFinishStep, phoneNumberNeeded)
                                .then(resolve)
                                .catch(reject);
                            break;
                        case otp_1.OtpFinishResultType.OnSuccess:
                            let input = result.input;
                            session
                                .fetchFromBackend('/v1/client/mobile/otp/finish', {
                                otp: (_a = input === null || input === void 0 ? void 0 : input.otp) !== null && _a !== void 0 ? _a : '',
                            })
                                .then((response) => {
                                const authResponse = response;
                                if (authResponse.error) {
                                    reject(new auth_error_1.default(authResponse.error.message, authResponse.error.code, response.next, false));
                                }
                                else if (authResponse.data) {
                                    let data = authResponse.data;
                                    var errorMessage = '';
                                    if (data === null || data === void 0 ? void 0 : data.code) {
                                        errorMessage += `Error Code: ${data.code}, `;
                                    }
                                    if (data === null || data === void 0 ? void 0 : data.message) {
                                        errorMessage += `${data.message}`;
                                    }
                                    else {
                                        errorMessage += `Error validating OTP`;
                                    }
                                    let otpError = new otp_1.OtpError(errorMessage, data === null || data === void 0 ? void 0 : data.code);
                                    this.log.error(`Server reports invalid OTP: ${errorMessage}`);
                                    this.runOtpFinishStep(session, otpStartStep, otpFinishStep, phoneNumberNeeded, otpError)
                                        .then(resolve)
                                        .catch(reject);
                                }
                                else {
                                    resolve(response.next);
                                }
                            })
                                .catch(reject);
                            break;
                        case otp_1.OtpFinishResultType.OnMobileNumberChange:
                            this.runOtpStartStep(session, otpStartStep, otpFinishStep, true)
                                .then(resolve)
                                .catch(reject);
                            break;
                    }
                }
            })
                .catch((error) => {
                const message = auth_error_1.default.extractMessage(error);
                reject(new Error(`Failed to obtain OTP for verification: ${message}}`));
            });
        });
    }
}
MobileOtpStep.NAME = 'mobile/otp';
exports.default = MobileOtpStep;
