"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const logger_1 = require("../common/logger");
const platform_1 = require("./platform");
const SIGNATURE_VERSION = '3';
const FUDGE_FACTOR = 60;
const SIGN_REQUESTS = [
    '/v1/client/stepup/unregister',
    '/v1/client/device/unregister',
    '/v1/client/device/passive/error',
    '/v1/client/mobile/instant/error',
    '/v1/client/mobile/otp/error',
    '/v1/client/user/mobileactive',
    '/v1/client/user/mobileactive/error',
];
class RequestSignerV3 {
    constructor(session) {
        this.log = logger_1.LoggerFactory.getLogger('request-signer');
        this.session = session;
    }
    sign(method, path, query, contentType, body) {
        return new Promise((resolve, reject) => {
            this.session
                .getDeviceRegistration()
                .then((registration) => {
                if (!registration || !registration.deviceId) {
                    this.log.debug('Device not registered, cannot sign');
                    resolve(null);
                    return;
                }
                if (!SIGN_REQUESTS.includes(path)) {
                    this.log.debug('No signing needed for ' + path);
                    resolve(null);
                    return;
                }
                Promise.all([this.getChallenge(registration.deviceId), this.getHash(body)])
                    .then((results) => {
                    const challenge = results[0];
                    const bodyHash = results[1];
                    const dataToSign = method +
                        '\n' +
                        path +
                        '\n' +
                        query +
                        '\n' +
                        contentType +
                        '\n' +
                        challenge +
                        '\n' +
                        bodyHash;
                    registration
                        .sign(dataToSign)
                        .then((signature) => resolve({
                        version: SIGNATURE_VERSION,
                        challenge: challenge,
                        signature: signature,
                        keyId: registration.keyId,
                    }))
                        .catch(reject);
                })
                    .catch(reject);
            })
                .catch(reject);
        });
    }
    getChallenge(deviceId) {
        return new Promise((resolve, reject) => {
            if (this.cachedChallenge &&
                this.cachedChallenge.receivedAt &&
                this.cachedChallenge.ttl &&
                this.cachedChallenge.deviceId === deviceId &&
                this.cachedChallenge.receivedAt + this.cachedChallenge.ttl - FUDGE_FACTOR > (0, platform_1.getUnixTime)()) {
                resolve(this.cachedChallenge.challenge);
                return;
            }
            this.session
                .fetchFromBackend('/v1/client/challenge', {
                deviceId: deviceId,
            })
                .then((response) => response)
                .then((response) => {
                var _a;
                if (response.error) {
                    reject((_a = response.error) === null || _a === void 0 ? void 0 : _a.message);
                }
                else {
                    response.receivedAt = (0, platform_1.getUnixTime)();
                    this.cachedChallenge = response;
                    resolve(response.challenge);
                }
            })
                .catch(reject);
        });
    }
    getHash(data) {
        return new Promise((resolve, reject) => {
            crypto.subtle
                .digest('SHA-256', (0, platform_1.stringToArrayBuffer)(data))
                .then((hash) => resolve((0, platform_1.arrayBufferToHexString)(hash)))
                .catch(reject);
        });
    }
}
exports.default = RequestSignerV3;
