"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebDeviceRegistration = void 0;
const logger_1 = require("../common/logger");
const auth_error_1 = __importDefault(require("./auth-error"));
const platform_1 = require("./platform");
class WebDeviceRegistration {
    constructor(options) {
        this.deviceId = null;
        this.namespace = options.namespace;
        this.endpoint = options.endpoint;
        if (options.createdAt) {
            this.createdAt = options.createdAt;
            this.keyId = options.keyId;
            this.deviceId = options.deviceId;
            this.algorithm = options.algorithm;
            this.keys = options.keys;
        }
        else {
            this.keyId = crypto.randomUUID();
            this.algorithm = 'ES256';
            this.createdAt = (0, platform_1.getUnixTime)();
        }
    }
    setFpSignal(fingerprintSignal) {
        this.fingerprint = fingerprintSignal;
    }
    sign(data) {
        return new Promise((resolve, reject) => {
            this.initialize()
                .then(() => {
                if (this.keys) {
                    crypto.subtle
                        .sign({
                        name: 'ECDSA',
                        hash: { name: 'SHA-256' },
                    }, this.keys.privateKey, (0, platform_1.stringToArrayBuffer)(data))
                        .then((signature) => resolve(btoa((0, platform_1.arrayBufferToString)(this.p1363ToDer(signature)))))
                        .catch(reject);
                }
            })
                .catch(reject);
        });
    }
    getPublicKey() {
        return new Promise((resolve, reject) => {
            this.initialize()
                .then(() => {
                if (this.keys) {
                    crypto.subtle
                        .exportKey('spki', this.keys.publicKey)
                        .then((key) => {
                        const encodedKey = btoa((0, platform_1.arrayBufferToString)(key));
                        resolve('-----BEGIN PUBLIC KEY-----\n' + encodedKey + '\n-----END PUBLIC KEY-----');
                    })
                        .catch(reject);
                }
            })
                .catch(reject);
        });
    }
    getAuthRegistration(challenge) {
        return new Promise((resolve, reject) => {
            this.sign(challenge)
                .then((signature) => {
                this.getPublicKey()
                    .then((key) => {
                    resolve({
                        publicKey: {
                            id: this.keyId,
                            alg: this.algorithm,
                            key: key,
                            uvLevel: 0,
                        },
                        signature: signature,
                    });
                })
                    .catch(reject);
            })
                .catch(reject);
        });
    }
    getSignals() {
        return {
            fingerprint: this.fingerprint,
        };
    }
    initialize() {
        return new Promise((resolve, reject) => {
            if (this.keys) {
                resolve();
            }
            else {
                crypto.subtle
                    .generateKey({ name: 'ECDSA', namedCurve: 'P-256' }, false, ['sign', 'verify'])
                    .then((keyPair) => {
                    this.keys = keyPair;
                    resolve();
                })
                    .catch(reject);
            }
        });
    }
    p1363ToDer(sig) {
        const signature = (0, platform_1.arrayBufferToHexString)(sig);
        let r = signature.substring(0, signature.length / 2);
        let s = signature.substring(signature.length / 2);
        r = r.replace(/^(00)+/, '');
        s = s.replace(/^(00)+/, '');
        if (parseInt(r.charAt(0), 16) >= 8)
            r = `00${r}`;
        if (parseInt(s.charAt(0), 16) >= 8)
            s = `00${s}`;
        const rString = `02${(r.length / 2).toString(16).padStart(2, '0')}${r}`;
        const sString = `02${(s.length / 2).toString(16).padStart(2, '0')}${s}`;
        const derSig = `30${((rString.length + sString.length) / 2)
            .toString(16)
            .padStart(2, '0')}${rString}${sString}`;
        const matchSig = derSig.match(/[\da-f]{2}/gi);
        return matchSig ? new Uint8Array(matchSig.map((h) => parseInt(h, 16))) : new Uint8Array();
    }
    lenVal(val) {
        if (typeof val === 'string' || val instanceof String) {
            const vs = val;
            return `[${vs.length}]: ${vs}`;
        }
        else {
            const ab = val;
            return `[${ab.byteLength}]: ${Array.from(new Uint8Array(ab), (x) => x.toString(16)).join(',')}`;
        }
    }
}
exports.WebDeviceRegistration = WebDeviceRegistration;
class WebDeviceAuth {
    constructor(dbFactory) {
        this.log = logger_1.LoggerFactory.getLogger('web-device-auth');
        this.dbFactory = dbFactory;
    }
    createRegistration(options) {
        return new Promise((resolve, reject) => {
            resolve(new WebDeviceRegistration(options));
        });
    }
    getRegistration(namespace) {
        return new Promise((resolve, reject) => {
            const openReq = this.openDatabase();
            openReq.onerror = (e) => {
                reject('Failed to open registration database: ' + e);
            };
            openReq.onsuccess = (e) => {
                const db = e.target.result;
                const tx = db.transaction([WebDeviceAuth.DB_STORE], 'readonly');
                tx.oncomplete = () => db.close();
                const store = tx.objectStore(WebDeviceAuth.DB_STORE);
                const getReq = store.get(namespace);
                getReq.onerror = (e) => {
                    this.log.error(e);
                    reject(new Error('Failed to access registration'));
                };
                getReq.onsuccess = () => {
                    if (getReq.result) {
                        resolve(new WebDeviceRegistration(getReq.result));
                    }
                    else {
                        resolve(null);
                    }
                };
            };
        });
    }
    storeRegistration(registration) {
        return new Promise((resolve, reject) => {
            const openReq = this.openDatabase();
            openReq.onerror = (e) => {
                reject('Failed to open registration database: ' + e);
            };
            openReq.onsuccess = (e) => {
                const db = e.target.result;
                const tx = db.transaction([WebDeviceAuth.DB_STORE], 'readwrite');
                tx.oncomplete = () => db.close();
                const store = tx.objectStore(WebDeviceAuth.DB_STORE);
                this.log.trace('Saving registration');
                const addReq = store.put(registration);
                addReq.onsuccess = () => {
                    this.log.trace('Registration saved');
                    resolve();
                };
                addReq.onerror = (e) => {
                    this.log.error(e);
                    reject(new auth_error_1.default('Failed to store registration'));
                };
            };
        });
    }
    deleteRegistration(namespace) {
        return new Promise((resolve, reject) => {
            var openReq = this.openDatabase();
            openReq.onerror = (e) => {
                reject('Failed to open registration database: ' + e);
            };
            openReq.onsuccess = (e) => {
                var db = e.target.result;
                var tx = db.transaction([WebDeviceAuth.DB_STORE], 'readwrite');
                tx.oncomplete = () => db.close();
                if (!db.objectStoreNames.contains(WebDeviceAuth.DB_STORE)) {
                    this.log.debug('Registration not found, store is missing');
                    resolve();
                    return;
                }
                var store = tx.objectStore(WebDeviceAuth.DB_STORE);
                var delReq = store.delete(namespace);
                delReq.onsuccess = () => {
                    resolve();
                };
                delReq.onerror = (e) => {
                    this.log.error(e);
                    reject(new Error('Failed to delete registration'));
                };
            };
        });
    }
    reset() {
        return new Promise((resolve, reject) => {
            var delReq = this.dbFactory.deleteDatabase(WebDeviceAuth.DB_NAME);
            delReq.onsuccess = (e) => {
                resolve();
            };
            delReq.onerror = (e) => {
                this.log.warn('Failed to delete registration database');
                this.log.warn(e);
                resolve();
            };
        });
    }
    openDatabase() {
        const openReq = this.dbFactory.open(WebDeviceAuth.DB_NAME, WebDeviceAuth.DB_VERSION);
        openReq.onupgradeneeded = (e) => {
            var _a;
            const db = (_a = e.target) === null || _a === void 0 ? void 0 : _a.result;
            if (db) {
                this.log.debug('Registration not found, database is missing, initializing');
                const store = db.createObjectStore(WebDeviceAuth.DB_STORE, { keyPath: 'namespace' });
                store.transaction.oncomplete = (e) => {
                    this.log.trace('Store initialization completed');
                };
            }
            else {
                this.log.error('Failed to initialize registration database: database is null');
            }
        };
        return openReq;
    }
}
WebDeviceAuth.DB_VERSION = 1;
WebDeviceAuth.DB_NAME = 'ProveAuth';
WebDeviceAuth.DB_STORE = 'Registrations';
exports.default = WebDeviceAuth;
